#include "main.h"
#include <math.h>

void send_reply(unsigned char type, unsigned int parameter)
{
	unsigned char temp = 0;
	unsigned char checksum = 0;
	char out_buffer[10];
	
	sprintf(out_buffer, "!%c%c%04X",bot_number, type, parameter);
	for(temp = 0; temp < 7; temp++)
	{
		checksum ^= out_buffer[temp];
	};

	FILE *prev_stdout;
	prev_stdout = stdout;
	stdout = &usart1;
	//we reroute the printf function onto the serial line one, and store the original stdout. 
	//at the end of the function, the stdout is restored to its previos value
	
	printf("!%c%c%04X%02X\n",bot_number,type,parameter,checksum);
	stdout = prev_stdout;
	return;
}

unsigned char get_number_from_packet_error;	//this will be set to 1, if we feed this function BS (not hex nubmers in the 3,4,5 and 6 slots
unsigned int get_number_from_packet(unsigned char *in)
{
	unsigned int out=0,multiplier = 0x01;
	unsigned char number,temp;
	get_number_from_packet_error = 0;
	for(temp = 0; temp < 4; temp ++)
	{
		if((number = hex_to_dec(in[6-temp]))==HEX_DEC_ERROR) get_number_from_packet_error = 1; 
		out |= number*multiplier;
		multiplier *=0x10;
	}
	return out;
}

static inline void analyze_packet(void)
{
	unsigned char internal_buffer[COM_BUFFER_LENGTH];
	unsigned char temp, checksum1, checksum2, checksum_real = 0;
	unsigned int number=0;
	
	for(temp = 0; temp!= COM_BUFFER_LENGTH; temp++) 
	{
		//In order to allow the interrupt to receive a new packet while we are 
		//analyzing the old one, we copy the old one to an internal buffer of the 
		//function. that way, the interrupt can store any new packet at any time
		//into the packet_buffer. 
		internal_buffer[temp] = packet_buffer[temp];
	};

	

	FILE *prev_stdout;
	prev_stdout = stdout;
	stdout = &usart1;
	//we reroute the printf function onto the serial line one, and store the original stdout. 
	//at the end of the function, the stdout is restored to its previos value

	for(temp = 0; temp < 7; temp++) checksum_real ^= internal_buffer[temp];

	checksum1 = hex_to_dec(internal_buffer[8]);
	checksum2 = hex_to_dec(internal_buffer[7]);


	//we extract the ckecksum values and check them, if they are valid hex nubmers
	if(checksum1 == HEX_DEC_ERROR || checksum2 == HEX_DEC_ERROR || checksum_real != (checksum2*0x10 | checksum1))
	{
		//if the ckecksum was invalid (as in not equal, or gibberish was entered insted of normal numbers
		//we send an error
		send_reply(REP_BAD_CHECKSUM,checksum_real);
	}
	else	//if the ckecksum was OK, we do the packet
	{
		switch(internal_buffer[2])	//here we try to find out what does the packet want from us
		{

			case COM_SET_LEDS: //here we set the LEDs
				number = get_number_from_packet(internal_buffer);
				if(number > 0x0F || get_number_from_packet_error)
				{
					send_reply(REP_INVALID_PARAMETER,0);
				}
				else
				{
					set_leds(number);
					send_reply(REP_OK,0);
				}
				break;



			case COM_IS_ALIVE:
				send_reply(REP_STAT_ALIVE,ROBOT_FW_VERSION);
				break;
			


			case COM_NUMBER_QUERRY:	//this command is most likely addresed to all robots, with * as the name.
				for(number = bot_number; number != 0; number--)wait(12);	//first we wait for everyone before us to transmit
				send_reply(REP_OK,ROBOT_FW_VERSION);	//then we say hello
				break;


			
			case COM_SENSOR_STAT_BUMPER:
				send_reply(REP_STAT_BUMPER,bumper_stat(BUMPER_ALL));
				break;


			
			case COM_SENSOR_STAT_ANALOG:
				number = get_number_from_packet(internal_buffer);
				if(number > 7 || get_number_from_packet_error)
				{
					send_reply(REP_INVALID_PARAMETER,0);
				}
				else
				{
					send_reply(REP_STAT_ANALOG,(number*0x1000) | adc_vals[number]);
				}
				break;



			case COM_FULL_STOP:
				set_servo(0,SERVO_STOP);
				servo_123_off();
				servo_456_off();
				ticks_until_stop = 0;
				send_reply(REP_OK,0);
				break;				


			
			case COM_TIMED_FORWARD:
				ticks_until_stop = get_number_from_packet(internal_buffer);
				if(get_number_from_packet_error)send_reply(REP_INVALID_PARAMETER,0);
				set_servo(1,SERVO_FW);
				set_servo(4,SERVO_BW);
				servo_123_on();
				servo_456_on();
				send_reply(REP_OK,0);
				break;
			
			
			
			case COM_TIMED_BACKWARD:
				ticks_until_stop = get_number_from_packet(internal_buffer);
				if(get_number_from_packet_error)send_reply(REP_INVALID_PARAMETER,0);
				set_servo(1,SERVO_BW);
				set_servo(4,SERVO_FW);
				servo_123_on();
				servo_456_on();
				send_reply(REP_OK,0);
				break;
			
			
			
			case COM_TIMED_LEFT:
				ticks_until_stop = get_number_from_packet(internal_buffer);
				if(get_number_from_packet_error)send_reply(REP_INVALID_PARAMETER,0);
				set_servo(1,SERVO_BW);
				set_servo(4,SERVO_BW);
				servo_123_on();
				servo_456_on();
				send_reply(REP_OK,0);
				break;
			
			
			
			case COM_TIMED_RIGHT:
				ticks_until_stop = get_number_from_packet(internal_buffer);
				if(get_number_from_packet_error)send_reply(REP_INVALID_PARAMETER,0);
				set_servo(1,SERVO_FW);
				set_servo(4,SERVO_FW);
				servo_123_on();
				servo_456_on();
				send_reply(REP_OK,0);
				break;
			

			
			case COM_CONTINUOUS_FORWARD:
				ticks_until_stop = TIMER_SERVO_CONTINUOUS_OPERATION;
				set_servo(1,SERVO_FW);
				set_servo(4,SERVO_BW);
				servo_123_on();
				servo_456_on();
				send_reply(REP_OK,0);
				break;



			case COM_CONTINUOUS_BACKWARD:
				ticks_until_stop = TIMER_SERVO_CONTINUOUS_OPERATION;
				set_servo(1,SERVO_BW);
				set_servo(4,SERVO_FW);
				servo_123_on();
				servo_456_on();
				send_reply(REP_OK,0);
				break;



			case COM_CONTINUOUS_LEFT:
				ticks_until_stop = TIMER_SERVO_CONTINUOUS_OPERATION;
				set_servo(1,SERVO_BW);
				set_servo(4,SERVO_BW);
				servo_123_on();
				servo_456_on();
				send_reply(REP_OK,0);
				break;



			case COM_CONTINUOUS_RIGHT:
				ticks_until_stop = TIMER_SERVO_CONTINUOUS_OPERATION;
				set_servo(1,SERVO_FW);
				set_servo(4,SERVO_FW);
				servo_123_on();
				servo_456_on();
				send_reply(REP_OK,0);
				break;



			case COM_SET_NUMBER:
				number = get_number_from_packet(internal_buffer);
				if(get_number_from_packet_error || (!isalpha(number) && !isdigit(number))) send_reply(REP_INVALID_PARAMETER,0);
				else
				{
					set_sbot_number(number);
					send_reply(REP_OK,bot_number);
					init_bluetooth();
					wdt_enable(WDTO_500MS);
				}
				break;



			default:	//if we got an unknown packet type. 
				send_reply(REP_UNKNOWN_COMMAND,0);
				break;
		}
	}




	stdout = prev_stdout;
	//here we restore the stdout to its previosu value 
	return;
}

void forward(){
	ticks_until_stop = TIMER_SERVO_CONTINUOUS_OPERATION;
	set_servo(1,SERVO_FW);
	set_servo(4,SERVO_BW);
	servo_123_on();
	servo_456_on();
}

void moreLeft(){
	ticks_until_stop = TIMER_SERVO_CONTINUOUS_OPERATION;
	set_servo(1,SERVO_STOP);
	set_servo(4,SERVO_BW);
	servo_123_on();
	servo_456_on();
}

void moreRight(){
	ticks_until_stop = TIMER_SERVO_CONTINUOUS_OPERATION;
	set_servo(1,SERVO_FW);
	set_servo(4,SERVO_STOP);
	servo_123_on();
	servo_456_on();
}


void stop(){
	set_servo(1,SERVO_STOP);
	set_servo(4,SERVO_STOP);
	servo_123_off();
	servo_456_off();
}


int getState(int ls,int rs){
	if(ls<920 && rs<920)
		return 0; //both white
	if(ls>=920 && rs<920)
		return 1; //left black, right white
	if(ls<920 && rs>=920)
		return 2; //left white, right black
	if(ls>=920 && rs>=920)
		return 3; //both black
	return 0;
}

double getRand(){
	return (double)rand()/(double)RAND_MAX;
}

///////////////////////////////////////////////////////////////////////////////
int main(void)
{

	io_init();


	set_leds(0x01);
	wait(100);
	set_leds(0x02);
	wait(100);
	set_leds(0x04);
	wait(100);
	set_leds(0x08);
	wait(100);
	set_leds(0x00);

	get_sbot_number();
	stdout = &usart1;	
	if(button_pressed(GP_BUTTON_0) && button_pressed(GP_BUTTON_1) && button_pressed(GP_BUTTON_2))init_bluetooth();

	double p;
	double alpha = 0.2;
	double pmin = 0.3;
	int o = 30;
	double rules[12];
	int r, rmax, wi =0;
	int rewards[o];
	int state, prev_state = 7;	

	int k;
	for(k=0;k<o;k++)
		rewards[k] = 0;
	for(k=0;k<12;k++)
		rules[k] = 0.001;

	while(!new_packet){
		printf("RL Sbot \n");
		wait(500);
	}

	while(1)
	{
		if(new_packet == 1)	//if we received a new packed ,we should take a look at it. 
		{
			new_packet = 0;
			switch(packet_buffer[0]){
			case 'a': printf("hello\n"); 
			          break;
            case 's': printf("%d %d %d %d\n",adc_vals[0],adc_vals[1],adc_vals[2],adc_vals[3]);
					  break;
			case 'f': forward(); 
					  printf("forward\n");
			          break; 
			case 'l': moreLeft(); 
					  printf("left\n");
			          break; 
			case 'r': moreRight(); 
					  printf("right\n");
			          break; 
			case 'u': prev_state = 7;
					  break;
			case 't': prev_state = 7;
					  break;

			case ' ': stop();
					  int i;
					  for(i=0;i<12;i++){
					  	//printf("%d ",rules[i]);

					  }
					  printf("\n");
					  break;	
			}
			
			if((packet_buffer[0]!='u') && (packet_buffer[0]!='t')){
			  	  while(!new_packet){
					wait(500);
				}
				continue;
			}
			//analyze_packet();
		}
  //printf(".\n");
		//podla vstupov zistim stav
		state = getState(adc_vals[0],adc_vals[1]);
		if(state == prev_state){
			wait(30);
			continue;
		}
		else
			prev_state = state;

		int offset = state*3;
		rmax = o;
		int rsum =0;
		int j;
		for(j=0;j<o;j++){ //spocitam rewards
			rsum += rewards[j];
		}

		//pravdepodobnost vybratia nahodnej akcie
		p = pmin + ((double)rsum/(double)rmax)*(1.0-pmin);
		
		//vyber akcie
		int action = 0;
		double rn = getRand();
		if(packet_buffer[0]=='t'){
			double max = 0;
			int i;
			for(i=0;i<3;i++){
				if(rules[offset+i]>max){
					max = rules[offset+i];
					action = i;
				}
			}			
		}
		else if(rn>p){
			//vyber akciu pre dany stav pomocou roulette wheel
			int i;
			double sum = 0;
			for(i=0;i<3;i++){ //prechadzam cez tri akcie pre dany stav
				sum += rules[offset+i];
			}
			double randVal = getRand()*sum;
			sum = 0;
			for(i=0;i<3;i++){
				sum += rules[offset+i];
				if(sum>=randVal) //ked prekroci nahodne vybratu hodnotu zo sumy, ziskali sme akciu
					action = i;
			}
			
		}
		else{
			//vyber nahodnu akciu pre dany stav
			action = floor(getRand()*3);
			printf("n");
		}
		
		printf("%d\n",action);
		//vykonam vybratu akciu
		switch(action){
			case 0 : forward();
					 //printf("FORWARD");
					 break;
			case 1 : moreLeft();
					 //printf("LEFT");
					 break;
			case 2 : moreRight();
					 //printf("RIGHT");
					 break;

		}
		
		if(packet_buffer[0]!='t'){
			//podla stredneho senzora zistim reward
			r = (adc_vals[2]>=850)?1:0;
			rewards[wi] = r;
			wi = (wi+1)%o;

			//aktualizujem hodnotu pravidla, ktore sme pouzili
			if(r==1){
				set_leds(0x01);
				rules[offset+action] = rules[offset+action]*(1-alpha)+alpha;
			}
			else{
				//zistim v akom stave sme po vykonani akcie
				set_leds(0x00);
				state = getState(adc_vals[0],adc_vals[1]);
				double max = 0;
				int i;
				for(i=0;i<3;i++){
					if(rules[state*3+i]>max)
						max = rules[state*3+i];
				}
				rules[offset+action] = max*alpha;
			}
		}		
		wait(80);
		
	}


}



///////////////////////////////////////////////////////////////////////////////

