#ifndef MAIN_H 
#define MAIN_H 

#define F_CPU 14745600UL

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <compat/deprecated.h>
#include <ctype.h>
#include <avr/wdt.h>


#include "misc.h"
#include "servo.h"
#include "adc.h"
#include "usart0.h"
#include "usart1.h"
#include "timers.h"
#include "protokol_defs.h"

int main(void);
/*
The main function. 
*/


static inline void io_init(void);
/*
inits all of the IO,ADCs, PWM generator...
ignore the bogus errors that the compiler warns about...
*/

///////////////////////////////////////////////////////////////////////////////

static inline void io_init(void)
{
	wdt_disable();
	DDRG = _BV(1);	//the decorational LEDs
	PORTG = _BV(0);//the port whose sole purpose is to have a gpio jumper

	DDRA = _BV(4) | _BV(5) | _BV(6) | _BV(7);
	PORTA = _BV(0) | _BV(1) | _BV(2) | _BV(3);
	//the port with the bumpers and the general leds
	
	DDRB = _BV(1) | _BV(2) | _BV(4) | _BV(5) | _BV(6) | _BV(7);	//set the outputs for the servo pwms. and the power
	PORTB = _BV(3) | _BV(4);	//enable the pullup on miso. adn disable power for  teh servos
	//here are the servos and the SPI bus. as of new, the PINB0 is connected to the LED on the Bluetooth
	//it makes it easier to determine wheter we are connected to a device
	
	DDRC = 0;//all of the GPIO digital pins are inputs by default
	PORTC = 0xFF;//enable pullups on all of them

	PORTD = _BV(4) | _BV(5) | _BV(6) | _BV(7);//the inputs for the GP buttons are inputs with pullups
	DDRD = _BV(3) | _BV(4);	//the TXD and reset for the bluetooth are outputs.
	
	DDRE = _BV(1) | _BV(2) | _BV(3) | _BV(4) | _BV(5);//enable the pwm signals for the second set of servos as outputs. and also the servo power enabler
	PORTE = _BV(0) | _BV(2) | _BV(6) | _BV(7);//the interrupts will have pullups. and servos are off by default

	DDRF = 0;//all the analog stuff is input. unfortunately, the avr does not incorporate a DAC
	PORTF = 0;//no pullups! in analog stuff it's catastrophy!


	// PWM mode 14 (fast PWM, TOP=ICR1)
	TCCR1A = (1 << WGM11);
	TCCR1B = ((1 << WGM13) | (1 << WGM12));
	TCCR3A = TCCR1A;
	TCCR3B = TCCR1B;

	// Start timers (prescale=8)
	TCCR1B |= (1 << CS11);
	TCCR3B |= (1 << CS11);

	// Clear OC1A/OC1B/OC1C pins on each timer
	TCCR1A |= ((1 << COM1A1) | (1 << COM1B1) | (1 << COM1C1));
	TCCR3A |= ((1 << COM3A1) | (1 << COM3B1) | (1 << COM3C1));

	// Set register values
	ICR1   = PWM_TOP;     // => 50Hz (T=20ms)
	ICR3   = PWM_TOP;     // => 50Hz (T=20ms)

	OCR1A  = SERVO_STOP;
	OCR1B  = SERVO_STOP;
	OCR1C  = SERVO_STOP;
	OCR3A  = SERVO_STOP;
	OCR3B  = SERVO_STOP;
	OCR3C  = SERVO_STOP;
	
	TCCR0 = _BV(CS00) | _BV(CS01) | _BV(CS02);	//the prescaller of the timer0 will be 1024, this will ensure we
									//have 100 ticks per second
	OCR0 = 0x8F;	//100 ticks per second

	TIMSK = _BV(OCIE0);
	
	//the settings for the ADC
	adc_select_ref(ADC_REF_256V);//set the reference of the ADC to 2.56V
	ADCSRA = _BV(ADEN) | _BV(ADIE) | _BV(ADPS2) | _BV(ADPS1) | _BV(ADPS0);
	//interupt enabled for the ADC, prescaler is 128
	start_adc();
	//we start the ADC

	
	UCSR0B = _BV(RXCIE0) | _BV(RXEN0) | _BV(TXEN0);
	UCSR0C = _BV(UCSZ01) | _BV(UCSZ00);
	UBRR0L = ((F_CPU / (16 * USART_0_BAUD)) - 1) & 0xFF; 

	UCSR1B = _BV(RXCIE1) | _BV(RXEN1) | _BV(TXEN1);
	UCSR1C = _BV(UCSZ11) | _BV(UCSZ10);
	UBRR1L = ((F_CPU / (16 * USART_1_BAUD)) - 1) & 0xFF; 
	//setting up the USARTs. 8 data, 1 stop, no parity. The baud rate is defined by 
	//the USART_0_BAUD definition somewhere above

	sei();
	//enable interrupts...
	return;
}	

///////////////////////////////////////////////////////////////////////////////


#endif
