#include "usart1.h"

/*
USART1 is the Bluetooth usart, through which the robot receives its commands from the Bluetooth master. 
*/


//when a new packet is received, this variable will be set to 1
volatile unsigned char new_packet = 0;
//when the whole packet is received, it will be stored here
volatile unsigned char packet_buffer[COM_BUFFER_LENGTH];


//////////////////////////////////////////////////////////////////////////////////////////////////////

FILE usart1 = FDEV_SETUP_STREAM(usart1_putchar, NULL, _FDEV_SETUP_WRITE);

//////////////////////////////////////////////////////////////////////////////////////////////////////


//if an interrupt occurs during the initialisation period of the lbuetooth it will be ignored
volatile unsigned char bluetooth_initing = 0;


unsigned char usart1_getchar(void) 	//wait for the char to be received and then return it
{
	loop_until_bit_is_set(UCSR1A,RXC);
	return UDR1;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////

int usart1_putchar(char c, FILE *stream)
{
	//(bluetooth_initing == 0 && !bluetooth_connection_established())return 0;
	
	if(c=='\n')usart1_putchar('\r',NULL);
	loop_until_bit_is_set(UCSR1A, UDRE);
	UDR1 = c;
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////

/*
THe buffer structure
0123456789 10

0 - !
1 - robot name
2 - command
3,4,5,6 - parameters
7,8 - checksum
9,10 - \n or \r or \r\n
*/

ISR(USART1_RX_vect)
{
	unsigned char a = usart1_getchar();
	new_packet = 1;
	packet_buffer[0] = a;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////

/*
The output vector
*/

ISR(USART1_TX_vect)
{
	return;
}



//////////////////////////////////////////////////////////////////////////////////////////////////////
void reset_bluetooth(void)
{
	cbi(PORTD,4);
	wait(1000);
	sbi(PORTD,4);

	return;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////

#define BT_INIT_NUBMER_OF_WAITS 2
#define BT_LED_DECO_TIME 150
#define BT_TIME_TO_WAIT 1000
void init_bluetooth(void)
{

	UBRR1L = ((F_CPU / (16 * 9600UL)) - 1) & 0xFF; //change the baud to the default one of the BT module
	reset_bluetooth();//make sure we are communicating with the bluetooth correctly
	
	bluetooth_initing = 1;

	stdout = &usart1;
	wait(BT_TIME_TO_WAIT);

	printf("AT\n");
	set_leds(0x01);
	wait(BT_TIME_TO_WAIT);
	
	set_leds(0x02);
	printf("AT+BTMODE,3\n");
	wait(BT_TIME_TO_WAIT);
	
	set_leds(0x04);
	printf("AT+BTNAME=\"SBOT%c\"\n",bot_number);
	wait(BT_TIME_TO_WAIT);

	set_leds(0x08);
	printf("AT+BTCANCEL\n");
	wait(BT_TIME_TO_WAIT);
	
	set_leds(0x08);
	printf("AT+BTSEC,0,0\n");
	wait(BT_TIME_TO_WAIT);
	
	set_leds(0x08);
	printf("AT+UARTCONFIG,115200,N,1,1\n");
	wait(BT_TIME_TO_WAIT);


	set_leds(0x08);
	printf("ATS10=0\n");
	wait(BT_TIME_TO_WAIT);


	printf("ATZ\n");
	set_leds(0x0F);
	wait(BT_TIME_TO_WAIT);

	set_leds(0x0F);
	wait(BT_LED_DECO_TIME);
	set_leds(0x0E);
	wait(BT_LED_DECO_TIME);
	set_leds(0x0C);
	wait(BT_LED_DECO_TIME);
	set_leds(0x08);
	wait(BT_LED_DECO_TIME);
	set_leds(0x00);
	wait(BT_LED_DECO_TIME);

	bluetooth_initing = 0;
	UBRR1L = ((F_CPU / (16 * USART_1_BAUD)) - 1) & 0xFF; //change the baud to the default one of the BT module
	return;
}

